/******************************************************/
/*         ROOMS - A room 'hanging' utility.          */
/*         V 1.7                                      */
/*         (c) October 1989. L.F. Holgate             */
/*                                                    */
/*         Room, object & vocab loading routines.     */
/*                                                    */
/******************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <string.h>
#include <dir.h>
#include <ctype.h>
#include "rooms17.h"
#include "player.h"

extern struct rm_file rooms[MAX_ROOMS];
extern struct vo_file vocab[MAX_WORDS];
extern struct obj objects[MAX_OBJECTS];

extern char *messages[MAX_MESSAGE];

void 	read_vocab(void),
	read_rooms(char *file_name),
	read_messages(void),
	get_room (FILE *fp, int room_num),
	read_objs(void),
	read_container(int obj_num, int flag),
	read_door(int obj_num),
	place_objs(void),
	init_rooms(void),
	init_messages(void),
	init_objs(void);

char 	*get_buffer(FILE *fp, int type);

struct player_idx *load_player(void);

struct player_idx *load_player(void)
{
	FILE *fp;
	char *temp, buffer[BUFFER_SIZE];
	struct player_idx	*first_player, *current_player, *prev_player;

	if ((fp = fopen("player.pla","r")) == NULL) {
		printf("Can't find player info file!\nPLAYER.PLA must be in this directory!\n");
		exit(20);
	}
	first_player = prev_player = NULL;
	strcpy(buffer,get_buffer(fp,3));
	while (strlen(buffer) >= 1) {
		if ((current_player = farmalloc(sizeof(struct player_idx))) == NULL) {
			printf("Error loading player info.");
			exit(20);
		}
		current_player->next_player = NULL;
		if (first_player == NULL)
			first_player = current_player;
		if (prev_player != NULL)
			prev_player->next_player = current_player;
		if ((temp = farmalloc(strlen(buffer)+1)) == NULL)
			error(9898);
		strcpy(temp,buffer);
		current_player->name = temp;
		strcpy(buffer,get_buffer(fp,3));
                if ((temp = farmalloc(strlen(buffer)+1)) == NULL)
			error(9898);
		strcpy(temp,buffer);
		current_player->title = temp;
		strcpy(buffer,get_buffer(fp,3));
		current_player->str = front_obj(buffer);
		current_player->dex = front_obj(buffer);
		current_player->con = front_obj(buffer);
		current_player->sta = front_obj(buffer);
		current_player->mag = front_obj(buffer);
		current_player->height = front_obj(buffer);
		current_player->weight = front_obj(buffer);
		current_player->sex = front_obj(buffer);
		current_player->race = front_obj(buffer);
		current_player->handedness = front_obj(buffer);
		current_player->class = front_obj(buffer);
		current_player->level = front_obj(buffer);
		current_player->magic_exp = atol(strtok(buffer,","));
		current_player->combat_exp = atol(strtok(NULL,","));
		current_player->general_exp = atol(strtok(NULL,","));
		prev_player = current_player;
		strcpy(buffer,get_buffer(fp,3));
	}
	fclose(fp);
	return(first_player);
}

void init_messages(void)
{
	int 	count;

	for(count = 0; count < MAX_MESSAGE;count++) {
		if (messages[count] != NULL && strlen(messages[count]) > 0) {
			farfree(messages[count]);
			messages[count] = NULL;
		}
	}
}

void read_messages(void)
{
	FILE 	*fp;
	char 	file_name[15], drive [MAXDRIVE], dir [MAXDIR], file[MAXFILE],
		ext [MAXEXT], buffer [BUFFER_SIZE];
	int	message_num = 1;

	strcpy(file_name,ROOM_FILE);
	fnsplit(file_name,drive,dir,file,ext);
	fnmerge(file_name,drive,dir,file,".mes");
	if ((fp = fopen(file_name, "r")) == NULL)
		return;
	for(;;) {
		strcpy (buffer,get_buffer(fp,LDESC));
		if (buffer [0] == '\0')
			break;
		if ((messages[message_num] = farmalloc(strlen(buffer)+1)) == NULL) {
			printf("No space for message %i!\n",message_num);
			exit(20);
		} else
			strcpy(messages[message_num],buffer);
		message_num++;
	}
	fclose(fp);
}

void read_vocab(void)
{
	FILE 	*fp;
	int	count, words, code[2];
	char	tmp [COMMAND_LENGTH];

	if ((fp = fopen("vocab.new","r")) == NULL) {
		puts("Can't find vocab file: Vocab.new!\nThis file must be located in the same directory as ROOMS.EXE.");
		exit(1);
	}
	fscanf(fp,"%d\n",&words);
	if (words > MAX_WORDS) {
		puts("Vocab file too big!");
		exit(1);
	}
	count = words;
	while (count) {
		fscanf(fp,"%s %d %d\n",&tmp,&code[0],&code[1]);
		if ((vocab [words - count].word = calloc(sizeof(char),strlen(tmp)+1)) == NULL)
			error(1);
		strcpy(vocab [words - count].word,tmp);
		vocab [words - count].command_code[0] = code[0];
		vocab [words - count].command_code[1] = code[1];
		--count;
		}
}

void read_rooms(char *file_name)
{
	FILE 	*fp;
	int	room_num;
	char    static	buffer [BUFFER_SIZE];

	if ((ROOM_FILE = (char *) farmalloc(strlen(file_name)+ 1)) == NULL)
		error(2);
	strcpy(ROOM_FILE,file_name);
	if ((fp = fopen(ROOM_FILE, "r")) == NULL) {
		printf("Can't find file: %s!",file_name);
		exit (1);
	}
	strcpy (buffer,get_buffer(fp,SDESC));
	if((ROOM_TITLE = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
		error(3);
	strcpy (ROOM_TITLE,buffer);
	strcpy (buffer,get_buffer(fp,SDESC));
	if((ROOM_TYPE = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
		error(4);
	strcpy (ROOM_TYPE,buffer);

	/* The above loads in the title string of the rooms and a flag that
	   determines how complex the room file is, i.e whether it includes
	   things like co-ordinates, night and day, darkness etc.

	   The following loop then loads the rooms into the room structure.*/

	for (room_num = 1; (get_room(fp,room_num), room_num <= MAX_ROOMS && (rooms [room_num].sdesc [0] != 0)); room_num++);
	rooms[room_num].sdesc = NULL;
	ROOM_NO = --room_num;
	fclose(fp);
}

void get_room (FILE *fp, int room_num)
{
	char	buffer [BUFFER_SIZE], *num_ptr;

	strcpy (buffer,get_buffer(fp,SDESC));
	if((rooms [room_num].sdesc = (char *) farmalloc(strlen(buffer)+1)) == NULL)
		error(5);
	strcpy (rooms [room_num].sdesc,buffer);
	if (strlen (rooms[room_num].sdesc) < 2)
		return;
	strcpy (buffer,get_buffer(fp,LDESC));
	if((rooms [room_num].ldesc = (char *) farmalloc(strlen(buffer)+1)) == NULL)
		error(6);
	strcpy (rooms [room_num].ldesc,buffer);
	strcpy (buffer,get_buffer(fp,EXITS));
	num_ptr = strtok (buffer,"|");
	if((rooms [room_num].exits = (char *) farmalloc(strlen(num_ptr)+1)) == NULL)
		error(7);
	strcpy (rooms [room_num].exits,num_ptr);
	num_ptr = strtok (NULL,"|");
	if (num_ptr == NULL)
		num_ptr = "";
	if((rooms [room_num].doors = (char *) farmalloc(strlen(num_ptr)+1)) == NULL)
		error(7);
	strcpy (rooms[room_num].doors,num_ptr);
	if((rooms[room_num].multi_room_objs = (char *) farmalloc(sizeof(char) * 2)) == NULL)
		error(7);
	strcpy (rooms[room_num].multi_room_objs,"");
	if (!strcmp(ROOM_TYPE,"NEW\n")) {
		strcpy(buffer,get_buffer(fp,SDESC));
		if((num_ptr = strtok(buffer,",")) == NULL)
			exit(1);
		rooms[room_num].x_coord = atoi(num_ptr);
		if((num_ptr = strtok(NULL,",")) == NULL)
			exit(1);
		rooms[room_num].y_coord = atoi(num_ptr);
		if((num_ptr = strtok(NULL,",")) == NULL)
			exit(1);
		rooms[room_num].z_coord = atoi(num_ptr);
	}
}

void read_objs(void)
{
	FILE 	*fp;
	char 	file_name[15], drive [MAXDRIVE], dir [MAXDIR], file[MAXFILE],
		ext [MAXEXT], buffer [BUFFER_SIZE], *num_ptr;
	int	obj_num = 1, tmp, count;

	OBJ_NO = 0;
	strcpy(file_name,ROOM_FILE);
	fnsplit(file_name,drive,dir,file,ext);
	fnmerge(file_name,drive,dir,file,".obd");
	if ((fp = fopen(file_name, "r")) == NULL)
		return;
	for(;;) {
		strcpy (buffer,get_buffer(fp,3));
		if (buffer [0] == '\0')
			break;
		if((objects [obj_num].name = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(8);
		strcpy (objects [obj_num].name ,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].desc = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(9);
		strcpy (objects [obj_num].desc,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].mdesc = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(10);
		strcpy (objects [obj_num].mdesc,buffer);
		strcpy (buffer,get_buffer(fp,LDESC));
		if((objects [obj_num].ldesc = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(9);
		strcpy (objects [obj_num].ldesc,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].aword = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(10);
		strcpy (objects [obj_num].aword,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].theword = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(11);
		strcpy (objects [obj_num].theword,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].noword = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(12);
		strcpy (objects [obj_num].noword,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].multi = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(10);
		strcpy (objects [obj_num].multi,buffer);
		strcpy (buffer,get_buffer(fp,3));
		if((objects [obj_num].type = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
			error(13);
                strcpy (objects [obj_num].type,buffer);
                strcpy (buffer,get_buffer(fp,3));
		num_ptr = strtok(buffer,",");
		objects [obj_num].feature = atoi(num_ptr);
		num_ptr = strtok(NULL,",");
		objects [obj_num].mid_desc = atoi(num_ptr);
                num_ptr = strtok(NULL,",");
		objects [obj_num].mass = atol(num_ptr);
                num_ptr = strtok(NULL,",");
		objects [obj_num].size = atoi(num_ptr);
		strcpy (buffer,get_buffer(fp,3));
		if ((num_ptr = strtok(buffer,",")) == NULL)
			exit(20);
		for(count = 0; count < 3; count++) {
			tmp = atoi(num_ptr);
			if (tmp == 1)
				read_container(obj_num,count);
			if ((num_ptr = strtok(NULL,",")) == NULL)
				break;
		}
                strcpy (buffer,get_buffer(fp,3));
		if ((num_ptr = strtok(buffer,",")) == NULL)
			exit(20);
		tmp = atoi(num_ptr);
		if (tmp == 1)
			read_door(obj_num);
                strcpy (buffer,get_buffer(fp,3));
		if ((num_ptr = strtok(buffer,",")) == NULL)
			exit(20);
		objects[obj_num].openable = atoi(num_ptr);
		if (objects[obj_num].openable) {
			if ((num_ptr = strtok(NULL,",")) == NULL)
				exit(1);
			objects[obj_num].open_state = atoi(num_ptr);
			num_ptr = strtok(NULL,",");
			objects[obj_num].lockable = atoi(num_ptr);
			num_ptr = strtok(NULL,",");
			objects[obj_num].lock_state = atoi(num_ptr);
		} else
			objects[obj_num].open_state = 1;
                strcpy (buffer,get_buffer(fp,3));
		objects[obj_num].loc_type = front_obj(buffer);
		if(objects[obj_num].loc_type != MULTI_ROOM)
			objects[obj_num].location = front_obj(buffer);
		else {
			if((objects [obj_num].multi_room = (char *) farmalloc(strlen(buffer)+ 1)) == NULL)
				error(106);
			strcpy (objects [obj_num].multi_room,buffer);
		}
		if (objects[obj_num].is_a_door != NULL)
			objects[obj_num].is_a_door->other_room = front_obj(buffer);
		obj_num++;
		if (obj_num > MAX_OBJECTS)
			break;
	}
	OBJ_NO = obj_num;
	place_objs();
}

void place_objs(void)
{
	char    *tmp_obj, multi_objs_list[COMMAND_LENGTH];
	int 	obj_num, room_num;

	for(obj_num = 1; obj_num <= OBJ_NO; obj_num++) {
		if (objects[obj_num].loc_type == ROOM) {
			if (objects[obj_num].is_a_door == NULL) {
				if (rooms [objects [obj_num].location].first_obj == 0) {
					rooms [objects [obj_num].location].first_obj = obj_num;
					objects [obj_num].prev_obj = 0;
				} else {
					objects [rooms[objects [obj_num].location].last_obj].next_obj = obj_num;
					objects [obj_num].prev_obj = rooms[objects [obj_num].location].last_obj;
				}
				rooms [objects [obj_num].location].last_obj = obj_num;
			} else {
				room_num = objects[obj_num].location;
				multi_objs_list[0] = 0;
				while (room_num) {
					strcpy(multi_objs_list,rooms[room_num].multi_room_objs);
					add_dig_to_string(multi_objs_list,obj_num,1);
                                        if (rooms[room_num].multi_room_objs != NULL)
						farfree(rooms[room_num].multi_room_objs);
					if ((rooms[room_num].multi_room_objs = (char *) farmalloc(strlen(multi_objs_list)+1)) == NULL)
						exit (9110);
					strcpy(rooms[room_num].multi_room_objs,multi_objs_list);
					if (room_num == objects[obj_num].is_a_door->other_room)
						room_num = 0;
					else
						room_num = objects[obj_num].is_a_door->other_room;
				}
			}
		} else if (objects[obj_num].loc_type == CONTAINER) {
					if (objects[objects [obj_num].location].contains_inside->first == 0) {
				objects[objects [obj_num].location].contains_inside->first = obj_num;
				objects [obj_num].prev_obj = 0;
			} else {
				objects [objects[objects [obj_num].location].contains_inside->last].next_obj = obj_num;
				objects [obj_num].prev_obj = objects[objects [obj_num].location].contains_inside->last;
			}
			objects[objects [obj_num].location].contains_inside->last = obj_num;
			objects[objects [obj_num].location].contains_inside->contained_wt += objects[obj_num].mass;
		} else if (objects[obj_num].loc_type == HELD) {
		} else if (objects[obj_num].loc_type == MULTI_ROOM) {
			if ((tmp_obj = (char *) farmalloc(strlen(objects[obj_num].multi_room)+1)) == NULL)
				exit(1212);
			strcpy(tmp_obj,objects[obj_num].multi_room);
			room_num = front_obj(tmp_obj);
			multi_objs_list[0] = 0;
			while (room_num) {
				strcpy(multi_objs_list,rooms[room_num].multi_room_objs);
				add_dig_to_string(multi_objs_list,obj_num,1);
				if (rooms[room_num].multi_room_objs != NULL)
					farfree(rooms[room_num].multi_room_objs);
				if ((rooms[room_num].multi_room_objs = (char *) farmalloc(strlen(multi_objs_list)+1)) == NULL)
					exit (9110);
				strcpy(rooms[room_num].multi_room_objs,multi_objs_list);
                        	room_num = front_obj(tmp_obj);
			}
                        farfree(tmp_obj);
		}
	}
}

void read_door(int obj_num)
{
	char	*num_ptr;
	int 	tmp;

	if ((objects[obj_num].is_a_door = (struct door_obj *) farmalloc (sizeof(struct door_obj))) == NULL) {
		printf("Error in allocating memory for a door object.\n");
		exit(1);
	}
        num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->bar_side = tmp;
	num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->open_side = tmp;
	num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->barable = tmp;
	num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->bar_state = tmp;
	num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->spikeable = tmp;
        num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->spike_state = tmp;
        num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->hidden_state = tmp;
        num_ptr = strtok(NULL,",");
	tmp = atoi(num_ptr);
	objects[obj_num].is_a_door->hidden_side = tmp;
}

void read_container(int obj_num, int flg)
{
	char	*num_ptr;
	int 	tmp;

	if (flg == 0) {
		if (objects[obj_num].contains_inside == NULL) {
			if ((objects[obj_num].contains_inside = (struct container_obj *) farmalloc (sizeof(struct container_obj))) == NULL) {
				printf("Error in allocating memory for a inside container object.\n");
				exit(1);
			return;
			}
		}
		num_ptr = strtok(NULL,",");
		objects[obj_num].contains_inside->max_wt = atol(num_ptr);
                num_ptr = strtok(NULL,",");
		tmp = atoi(num_ptr);
		objects[obj_num].contains_inside->max_siz = tmp;
                num_ptr = strtok(NULL,",");
		tmp = atoi(num_ptr);
		objects[obj_num].contains_inside->max_obj = tmp;
		objects[obj_num].contains_inside->contained_wt = 0;
		objects[obj_num].contains_inside->contained_siz = 0;
		objects[obj_num].contains_inside->contained_obj = 0;
		objects[obj_num].contains_inside->first = 0;
		objects[obj_num].contains_inside->last = 0;
	} else if (flg == 1) {
		if (objects[obj_num].contains_on_top == NULL) {
			if ((objects[obj_num].contains_on_top= (struct container_obj *) farmalloc (sizeof(struct container_obj))) == NULL) {
				printf("Error in allocating memory for a on_top container object.\n");
				exit(1);
			return;
			}
		}
	} else if (flg == 1) {
		if (objects[obj_num].contains_below == NULL) {
			if ((objects[obj_num].contains_below = (struct container_obj *) farmalloc (sizeof(struct container_obj))) == NULL) {
				printf("Error in allocating memory for a below container object.\n");
				exit(1);
			return;
			}
		}
	}
}

void init_rooms(void)
{
	int 	room_num = 1,
		end;

	if (!strcmp(ROOM_FILE,NULL))
		return;
	end = ROOM_NO;
	while (room_num <= end) {
		if (rooms[room_num].sdesc != NULL) {
			farfree(rooms[room_num].sdesc);
			rooms[room_num].sdesc = NULL;
			if (rooms[room_num].ldesc != NULL)
				farfree(rooms[room_num].ldesc);
			rooms[room_num].ldesc = NULL;
			if (rooms[room_num].exits != NULL)
				farfree(rooms[room_num].exits);
			rooms[room_num].exits = NULL;
			if (rooms[room_num].doors != NULL)
				farfree(rooms[room_num].doors);
			rooms[room_num].doors = NULL;
			if (rooms[room_num].multi_room_objs != NULL)
				farfree(rooms[room_num].multi_room_objs);
			rooms[room_num].multi_room_objs = NULL;
			rooms[room_num].x_coord = 0;
			rooms[room_num].y_coord = 0;
			rooms[room_num].z_coord = 0;
			rooms[room_num].first_obj = 0;
			rooms[room_num].last_obj = 0;
		}
		++room_num;
	}
}

void init_objs(void)
{
	int 	obj_num = 1;
	long	end;

	end = OBJ_NO;
	while (obj_num <= end) {
		if (objects[obj_num].name != NULL)
			farfree(objects[obj_num].name);
		objects[obj_num].name = NULL;
		if (objects[obj_num].desc != NULL)
			farfree(objects[obj_num].desc);
		objects[obj_num].desc = NULL;
		if (objects[obj_num].mdesc != NULL)
			farfree(objects[obj_num].mdesc);
		objects[obj_num].mdesc = NULL;
		if (objects[obj_num].aword != NULL)
			farfree(objects[obj_num].aword);
		objects[obj_num].aword = NULL;
		if (objects[obj_num].theword != NULL)
			farfree(objects[obj_num].theword);
		objects[obj_num].theword = NULL;
		if (objects[obj_num].noword != NULL)
			farfree(objects[obj_num].noword);
		objects[obj_num].noword = NULL;
		if (objects[obj_num].multi != NULL)
			farfree(objects[obj_num].multi);
		objects[obj_num].multi = NULL;
		if (objects[obj_num].type != NULL)
			farfree(objects[obj_num].type);
		objects[obj_num].type = NULL;
		if (objects[obj_num].multi_room != NULL)
			farfree(objects[obj_num].multi_room);
		objects[obj_num].multi_room = NULL;
		objects[obj_num].location = 0;
		objects[obj_num].mass = 0;
		objects[obj_num].size = 0;
		objects[obj_num].loc_type = 0;
		objects[obj_num].prev_obj = 0;
		objects[obj_num].next_obj = 0;
		objects[obj_num].feature = 0;
		objects[obj_num].mid_desc = 0;
		if (objects[obj_num].contains_inside != NULL)
			farfree(objects[obj_num].contains_inside);
		objects[obj_num].contains_inside = NULL;
		if (objects[obj_num].contains_on_top != NULL)
			farfree(objects[obj_num].contains_on_top);
		objects[obj_num].contains_on_top = NULL;
		if (objects[obj_num].contains_below != NULL)
			farfree(objects[obj_num].contains_below);
		objects[obj_num].contains_below = NULL;
		if (objects[obj_num].is_a_door != NULL)
			farfree(objects[obj_num].is_a_door);
		objects[obj_num].is_a_door = NULL;
		++obj_num;
	}
}

char *get_buffer(FILE *fp, int type)
{
	int     c, flg = 0, i;
	char 	static buffer [BUFFER_SIZE],
		stop;

	if (type == SDESC)
		flg = 1;
	if (type != LDESC)
        	stop = '\n';
	else
		stop = '*';
	i  = 0;
	while ((c = getc (fp)) != stop && (c != EOF) && (i < (BUFFER_SIZE - 1))) {
		if (c >= 128) {
			c = c - 128;
		}
		if (!(type == LDESC && isspace(c) && c != ' ')) {
			buffer [i] = c;
			++i;
		}
	}
	if (type == LDESC) {
		getc (fp);
	}
	if (type == 10 && i > 0) {
		buffer [i] = ',';
		++i;
	}
	if (flg && i > 1) {
		buffer [i] = '\n';
		++i;
	}
	buffer [i] = '\0';
	if (i == (BUFFER_SIZE + 1)) {
		puts("Buffer overflow!");
		printf("Error code: %d",type);
		getchar();
		exit (1);
	}
	return (buffer);
}
