/******************************************************/
/*         ROOMS - A room 'hanging' utility.          */
/*         V 1.7                                      */
/*         (c) October 1989. L.F. Holgate             */
/*                                                    */
/*         Object manipulation routines 2.            */
/*         Miscellaneous object routines.             */
/*                                                    */
/******************************************************/

#include <stdio.h>
#include <alloc.h>
#include <ctype.h>
#include <string.h>
#include "rooms17.h"
#include "player.h"

extern struct 	rm_file 	rooms[MAX_ROOMS];
extern struct	vo_file 	vocab[MAX_WORDS];
extern struct 	obj 		objects[MAX_OBJECTS];
extern struct 	player_info 	player, *player_ptr;
extern struct   flgs		flag, *flg_ptr;
extern struct   inv		inventory, *inv_ptr;

extern 	char 	*cap_first(char *in_file),
		*word_wrap(char *in_file),
		*is_it_here(int type,char classes[][5][COMMAND_LENGTH],int known[][5],int near_miss[][5],int obj_ref[],int obj_num[]);

void    add_dig_to_string(char *in_string,int dig,int comma),
	show_inv(struct flgs *flg_ptr, struct inv *inv_ptr),
	manip_type(int manip,int flag);

char    *make_list(int start),
	*count_obj(char *object_list, int flg),
	*dig_to_word(int digit),
	*show_obj(char all[],char list[],char *buffer,int flag,int or);

int 	front_obj(char in_string[]),
	count_them(char *list),
	manip_all(char all[], int obj_num, int count);

char *make_list(int start)
{
	/*******************************************************************/
	/* Makes a comma delimited string of objects, starting at START.   */
	/*******************************************************************/

	char 	list[OBJ_STR_LENGTH];
	int	obj_num = start;

	list[0] = 0;
	while (obj_num) {
		add_dig_to_string(&list[0],obj_num,1);
		obj_num = objects[obj_num].next_obj;
	}
	return (list);
}

void add_dig_to_string(char *in_string, int dig, int comma)
{
	/*******************************************************************/
	/* Adds DIG to the string IN_STRING and adds a comma after it if   */
	/* the flag COMMA is true.                                         */
	/*******************************************************************/

	char 	string[10];

	itoa(dig,&string[0],10);
	strcat(in_string,string);
	if (comma)
		strcat(in_string,",");
}

char *count_obj(char *object_list, int flg)
{
	/*******************************************************************/
	/* Looks through the string OBJECT_LIST for similar objects and    */
	/* returns a list consisting of the an object number of the type   */
	/* and the number of them. 					   */
	/*******************************************************************/

	int	count, flg2 = flg, obj_num, tmp_obj_num;
	char	list[OBJ_STR_LENGTH], *tmp_obj_list, *obj_list, *obj;

	list[0] = 0;
	obj_list = strdup(object_list);
	obj_num = front_obj(obj_list);
	while (obj_num) {
		count = 1;
		tmp_obj_list = strdup(obj_list);
		obj_list[0] = 0;
		obj = strtok(tmp_obj_list,",");
		tmp_obj_num = atoi(obj);
		while(tmp_obj_num) {
			if (flg == SDESC)
				flg2 = (objects[tmp_obj_num].mdesc[0] != NULL);
			else if (flg == WEIGH_OBJ)
				flg2 = ((objects[tmp_obj_num].mass + objects[tmp_obj_num].contains_inside->contained_wt) == (objects[obj_num].mass + objects[obj_num].contains_inside->contained_wt));
			else if (flg == WHERE_OBJ)
				flg2 = ((objects[tmp_obj_num].loc_type == objects[obj_num].loc_type) && (objects[tmp_obj_num].location == objects[obj_num].location));
			if (flg != WHERE_OBJ && (tmp_obj_num != obj_num) && (!strcmp(objects[obj_num].noword,objects[tmp_obj_num].noword)) && flg2)
				count++;
			else if (flg == WHERE_OBJ && (tmp_obj_num != obj_num) && flg2)
				count++;
			else {
				strcat(obj_list,obj);
				strcat(obj_list,",");
			}
			obj = strtok(NULL,",");
			tmp_obj_num = atoi(obj);
		}
		free(tmp_obj_list);
		add_dig_to_string(list,obj_num,1);
		add_dig_to_string(list,count,1);
		obj_num = front_obj(obj_list);
	}
	free(obj_list);
	return(list);
}

int front_obj(char in_string[])
{
	/*******************************************************************/
	/* Takes <in_string> and removes the first comma delimited number  */
	/* from it. Removes this number from the front of the list and     */
	/* returns the number, and the list.                               */
	/*******************************************************************/

	int 	obj_num = 0, length;
	char 	*obj;

	if ((obj = strtok(in_string,",")) != NULL) {
		obj_num = atoi(obj);
		length = strlen(obj);
		in_string[length] ='|';
		if (strlen(in_string) > length + 1) {
			in_string[length] ='|';
			strrev(in_string);
			strcpy(in_string,strtok(in_string,"|"));
			strrev(in_string);
		} else
			in_string[0] = '\0';
	}
	return(obj_num);
}

void manip_type(int manip,int flag)
{
        /*******************************************************************/
	/* Displays the correct word or phrase depending on the values of  */
	/* MANIP [the manipulation type] and FLAG [a flag distinguishing   */
	/* which word or phrase is required.                               */
	/*******************************************************************/

	if (flag == 1 && manip == TAKE_OBJ)
		printf("take");
	else if (flag == 2 && manip == TAKE_OBJ)
		printf("remove");
	else if (flag == 3 && manip == TAKE_OBJ)
		printf(" from");
	else if (flag == 4 && manip == TAKE_OBJ)
		printf("get something from");
	else if (flag == 1 && manip == DROP_OBJ)
		printf("drop");
	else if (flag == 2 && manip == DROP_OBJ)
		printf("insert");
	else if (flag == 3 && manip == DROP_OBJ)
		printf(" into");
	else if (flag == 4 && manip == DROP_OBJ)
		printf("put something into");
	else if (flag == 1 && manip == OPEN_OBJ)
		printf("open");
	else if (flag == 1 && manip == OPEN_OBJ)
		printf(" with");
	else if (flag == 1 && manip == CLOSE_OBJ)
		printf("close");
	else if (flag == 3 && manip == CLOSE_OBJ)
		printf(" with");
	else if (flag == 1 && manip == EMPTY_OBJ)
		printf("empty");
	else if (flag == 4 && manip == EMPTY_OBJ)
		printf("empty something into");
	else if (flag == 1 && manip == WHERE_OBJ)
		printf("locate");
	else if (flag == 1 && manip == WEIGH_OBJ)
		printf("weigh");
	else if (flag == 1 && manip == EXAMINE_OBJ)
		printf("examine");
	else
		printf(" UNKNOWN MANIPULATION.\nMANIP = %i\nFLAG = %i\nManip_type() needs updating. ",manip,flag);
}

char *dig_to_word(int digit)
{
	char 	word[COMMAND_LENGTH];

	word[0] = 0;

	if (digit == 0)
		strcat (word,"zero");
	if (digit > 99) {
		add_dig_to_string(word,digit,0);
		digit = 0;
	} else if (digit >= 90) {
		strcat (word,"ninety");
		digit -= 90;
                if (digit)
			strcat (word," ");
	} else if (digit >= 80) {
		strcat (word,"eighty");
		digit -= 80;
                if (digit)
			strcat (word," ");
	} else if (digit >= 70) {
		strcat (word,"seventy");
		digit -= 70;
		if (digit)
			strcat (word," ");
	} else if (digit >= 60) {
		strcat (word,"sixty");
		digit -= 60;
		if (digit)
			strcat (word," ");
	} else if (digit >= 50) {
		strcat (word,"fifty");
		digit -= 50;
		if (digit)
			strcat (word," ");
	} else if (digit >= 40) {
		strcat (word,"forty");
		digit -= 40;
		if (digit)
			strcat (word," ");
	} else if (digit >= 30) {
		strcat (word,"thirty");
		digit -= 30;
		if (digit)
			strcat (word," ");
	} else if (digit >= 20) {
		strcat (word,"twenty");
		digit -= 20;
		if (digit)
			strcat (word," ");
	}
	switch (digit) {
	case 1 :
		strcat(word,"one");
		break;
	case 2 :
		strcat(word,"two");
		break;
	case 3 :
		strcat(word,"three");
		break;
	case 4 :
		strcat(word,"four");
		break;
	case 5 :
		strcat(word,"five");
		break;
	case 6 :
		strcat(word,"six");
		break;
	case 7 :
		strcat(word,"seven");
		break;
	case 8 :
		strcat(word,"eight");
		break;
	case 9 :
		strcat(word,"nine");
		break;
	case 10 :
		strcat(word,"ten");
		break;
	case 11 :
		strcat(word,"eleven");
		break;
	case 12 :
		strcat(word,"twelve");
		break;
	case 13 :
		strcat(word,"thirteen");
		break;
	case 14 :
		strcat(word,"fourteen");
		break;
	case 15 :
		strcat(word,"fifteen");
		break;
	case 16 :
		strcat(word,"sixteen");
		break;
	case 17 :
		strcat(word,"seventeen");
		break;
	case 18 :
		strcat(word,"eighteen");
		break;
	case 19 :
		strcat(word,"nineteen");
		break;
	}
	return(word);
}

void show_inv(struct flgs *flg_ptr, struct inv *inv_ptr)
{
	int 	 obj_num;

	char	list[OBJ_STR_LENGTH], *buffer, *tmp_containers,
		containers[OBJ_STR_LENGTH], object_list[OBJ_STR_LENGTH],
		*capped;

	if (inv_ptr->first == 0) {
		puts ("You're not carrying anything!");
		flg_ptr->prompt = YES;
		return;
	}
	strcpy(object_list,make_list(inv_ptr->first));
	strcpy(list,count_obj(object_list,1));
        if((buffer = farmalloc ((BUFFER_SIZE * 2))) == NULL)
		error(502);
	buffer[0] = 0;
	strcat (buffer, "You are carrying the following: ");
	strcpy(containers,show_obj("",list,buffer,0,0));
	strcat(buffer,".\n");
	printf("%s",word_wrap(buffer));
	if (strlen(containers) != 0) {
		if ((tmp_containers = farmalloc(strlen(containers)+1)) == NULL)
			error(3010);
		strcpy(tmp_containers,containers);
	} else {
		if ((tmp_containers = farmalloc(strlen(" "))) == NULL)
			error(3010);
		strcpy(tmp_containers,"");
	}
	obj_num = front_obj(tmp_containers);
	while (obj_num) {
		strcpy(object_list,make_list(objects[obj_num].contains_inside->first));
		strcpy(list,count_obj(object_list,1));
		strcpy(buffer,"       ");
		strcat(buffer,capped = cap_first(objects[obj_num].theword));
		free(capped);
		if (objects[obj_num].open_state) {
			strcat(buffer," contains ");
			strcpy(containers,show_obj("",list,buffer,0,0));
                        strcat(buffer,".\n");
		} else
			strcat(buffer," is closed, you cannot see into it.\n");
		printf("%s",word_wrap(buffer));
		obj_num = front_obj(tmp_containers);
	}
	farfree(tmp_containers);
	farfree(buffer);
	flg_ptr->prompt = YES;
}

int manip_all(char all[], int obj_num, int count)
{
	int 	match_obj_num, match_count;
	char 	*tmp_all;

	if((tmp_all = farmalloc(strlen(all)+1)) == NULL)
		error(2020);
	strcpy(tmp_all,all);
	all[0] = 0;
	do {
		match_obj_num = front_obj(tmp_all);
		match_count = front_obj(tmp_all);
		if (match_obj_num == obj_num) {
			strcat(all,tmp_all);
			break;
		} else {
			add_dig_to_string(all,match_obj_num,1);
			add_dig_to_string(all,match_count,1);
		}
	} while (match_obj_num);
	if (match_count == count)
		return(0);
	else
		return(match_count);
}

int count_them(char *list)
{
	char 	*tmp_list;
	int	count;

	if((tmp_list = farmalloc (strlen(list)+1)) == NULL)
		error(505);
	strcpy(tmp_list,list);
	count = 1;
	if ((strtok(tmp_list,",")) == NULL)
		exit(2);
	if ((strtok(NULL,",")) == NULL)
		exit(2);
	for(;;) {
		if ((strtok(NULL,",")) == NULL)
			break;
		if ((strtok(NULL,",")) == NULL)
			break;
		count++;
	}
        farfree(tmp_list);
	return(count);
}

char *show_obj(char all[],char list[],char *buffer,int flag,int or)
{
	int	count, obj_num, num_objs, out_of;
	char	containers [OBJ_STR_LENGTH];

	containers[0] = 0;
	num_objs = count_them(list);
	obj_num = front_obj(list);
	count = front_obj(list);
	num_objs--;
	while(obj_num) {
        	out_of = 0;
		if (strlen(all) > 1)
			out_of = manip_all(all,obj_num,count);
		if(objects[obj_num].contains_inside != NULL && objects[obj_num].contains_inside->first != 0)
			add_dig_to_string(containers,obj_num,1);
		if (objects[obj_num].contains_inside != NULL && objects[obj_num].contains_inside->first== 0 && objects[obj_num].open_state) {
			if (flag)
				strcat(buffer,"the ");
			else
				strcat(buffer,"an ");
			strcat(buffer,"empty ");
			strcat(buffer,objects[obj_num].noword);
		} else if (!out_of && count == 1)
			if (flag)
				strcat(buffer,objects[obj_num].theword);
			else
				strcat(buffer,objects[obj_num].aword);
		else {
			if (flag && !out_of)
				strcat(buffer,"the ");
			strcat(buffer,dig_to_word(count));
			if (out_of) {
				strcat(buffer," of the ");
				strcat(buffer,dig_to_word(out_of));
			}
			strcat(buffer," ");
			if (objects[obj_num].multi[0] != 0)
				strcat(buffer,objects[obj_num].multi);
			else {
				strcat(buffer,objects[obj_num].noword);
				strcat(buffer,"s");
			}
		}
                obj_num = front_obj(list);
		count = front_obj(list);
		num_objs--;
		if (obj_num){
			if (num_objs)
				strcat(buffer, ", ");
			else
				if (or)
					strcat(buffer," or ");
				else
					strcat(buffer," and ");
		}
	}
	return (containers);
}