/******************************************************/
/*         ROOMS - A room 'hanging' utility.          */
/*         V 1.7                                      */
/*         (c) October 1989. L.F. Holgate             */
/*                                                    */
/*         Object manipulation routines 3.            */
/*         Examine, Weigh, Where.                     */
/*						      */
/******************************************************/

#include <stdio.h>
#include <alloc.h>
#include <ctype.h>
#include <string.h>
#include "rooms17.h"
#include "player.h"

extern struct 	rm_file 	rooms[MAX_ROOMS];
extern struct	vo_file 	vocab[MAX_WORDS];
extern struct 	obj 		objects[MAX_OBJECTS];
extern struct 	player_info 	player, *player_ptr;
extern struct   flgs		flag, *flg_ptr;
extern struct   inv		inventory, *inv_ptr;

extern 	char 	*cap_first(char *in_file),
		*word_wrap(char *in_file),
		*is_it_here(int type,char classes[][5][COMMAND_LENGTH],int known[][5],int near_miss[][5],int obj_ref[],int obj_num[]),
		*dig_to_word(int digit),
                *make_list(int start),
		*count_obj(char object_list[], int flg),
		*show_obj(char all[],char list[],char *buffer,int flag,int or);

void    examine_object(struct flgs *flg_ptr, struct inv *inv_ptr),
	weigh_object(struct flgs *flg_ptr, struct inv *inv_ptr),
	where_object(struct flgs *flg_ptr, struct inv *inv_ptr, struct player_info *player_ptr);

int	count_all(char *list);

char 	*show_mass(long mass);

void examine_object(struct flgs *flg_ptr, struct inv *inv_ptr)
{
	/*******************************************************************/
	/* Examines an object, or group of objects and displays the long   */
	/* description of the object if there is one available.            */
	/*******************************************************************/

	int 	obj_num[2], obj_ref[2], misc[2],
		near_miss[2][5], known[2][5], obj_count, none_examined = YES;
	char 	classes[2][5][COMMAND_LENGTH], examine_these[OBJ_STR_LENGTH],
		buffer[OBJ_STR_LENGTH * 2], list[OBJ_STR_LENGTH],
		*tmp_desc, *ldesc, *held, *inside, *capped;

	buffer [0] = 0;
	examine_these [0] = 0;
	object_manip(EXAMINE_OBJ,flg_ptr,inv_ptr,known,near_miss,classes,obj_num,obj_ref,examine_these,misc);
	if (misc[0] == -1)
		return;
	obj_count = misc[1];
	obj_num[OBJECT] = front_obj(examine_these);
	while (obj_num[OBJECT]) {
		if (strlen(objects[obj_num[OBJECT]].ldesc) > 2) { /* Do we have a long description? */
			if ((tmp_desc = farmalloc(strlen(objects[obj_num[OBJECT]].ldesc))) == NULL)
				exit(909);
			strcpy(tmp_desc,objects[obj_num[OBJECT]].ldesc);
			ldesc = strtok(tmp_desc,"|");                          /* Split off the long description part of the string. */
			held = strtok(NULL,"|");                               /* This bit is shown if you are holding the object. */
			inside = strtok(NULL,"|");                             /* This bit if you can see inside the object. */
			strcpy(buffer,ldesc);
			if (objects[obj_num[OBJECT]].loc_type == HELD && held)
				strcat(buffer,held);
		} else {
			strcpy(buffer,capped = cap_first(objects[obj_num[OBJECT]].theword));
			free(capped);
			strcat(buffer, " look");                               /* If there isn't a long description */
			if (strlen(objects[obj_num[OBJECT]].multi) < 2)        /* then tell them that the object looks */
				strcat(buffer,"s");                            /* just how they would expect it to look. */
			strcat(buffer," just how you would expect ");
			if (strlen(objects[obj_num[OBJECT]].multi) > 1)
				strcat(buffer,"them" );                        /* A multi [plural] object. */
			else
				strcat(buffer,"it");                           /* A singular object. */
			strcat(buffer," to look.");
		}
		if (objects[obj_num[OBJECT]].contains_inside) {                /* Is the object a container? */
			if (objects[obj_num[OBJECT]].open_state && inside)     /* Is it open? Does it have an inside description? */
				strcat(buffer,inside);                         /* Yes? Add it to the buffer then. */
			strcat(buffer," ");
			strcat(buffer,capped = cap_first(objects[obj_num[OBJECT]].theword));
			free(capped);
			if (objects[obj_num[OBJECT]].open_state) {
				if (objects[obj_num[OBJECT]].contains_inside->first) {
					strcpy(list,make_list(objects[obj_num[OBJECT]].contains_inside->first));
					strcpy(list,count_obj(list,1));
					strcat(buffer," contains ");           /* Count up what is inside */
					show_obj("",list,buffer,0,0);          /* and display it all nice. */
					strcat(buffer,".\n");
				} else
					strcat(buffer," is empty.\n");         /* Or don't. */
			} else
                        	strcat(buffer," is closed, you cannot see into it.\n");
		} else
			strcat(buffer,"\n\0");
		printf("%s",word_wrap(buffer));                                /* Print it all out to the screen. */
		obj_num[OBJECT] = front_obj(examine_these);                    /* Next object please. */
		none_examined = NO;                                            /* Yes, we've looked at one */
		--obj_count;
		if (obj_count == 0)                                            /* Have we examined as many as required? */
			break;                                                 /* if so, stop here. */
	}
	flg_ptr->prompt = YES;
	if (!none_examined)
		return;
	manip_error(EXAMINE_OBJ,OBJECT,classes,known,near_miss,obj_ref);       /* If nothing was weighed, display an error message. */
}

char *show_mass(long mass)
{
	/*******************************************************************/
	/* Returns a pointer to a string containing MASS formatted into    */
	/* kilograms and grams.                                            */
	/*******************************************************************/

	int 	mass_big, mass_small;
	char 	buffer[COMMAND_LENGTH];
	long	tmp_mass;

	buffer[0] = 0;
	tmp_mass = mass/1000;
	mass_big = (int) tmp_mass;
	tmp_mass = mass%1000;
	mass_small = (int) tmp_mass;
	if (mass_big || (!mass_big && !mass_small)) {
		/*strcat(buffer,dig_to_word(mass_big));*/                      /*Say it in words.*/
		add_dig_to_string(buffer,mass_big,0);                          /*Say it in digits.*/
		strcat(buffer," kilogram");
		if (mass_big != 1)
			strcat(buffer,"s");
	}
	if (mass_small) {
		if (mass_big)
			strcat(buffer," and ");
		/*strcat(buffer,dig_to_word(mass_small));/*                    /*Say it in words.*/
		add_dig_to_string(buffer,mass_small,0);                        /*Say it in digits.*/
		strcat(buffer," gram");
		if (mass_small > 1)
			strcat(buffer,"s");
	}
	return(buffer);
}

void weigh_object(struct flgs *flg_ptr, struct inv *inv_ptr)
{
	/*******************************************************************/
	/* Displays the weights of an object, or group of objects and then */
	/* the combined weight of all the objects weighed.                 */
	/*******************************************************************/

	long  	mass, total_mass = 0;
	int 	obj_num[2], obj_ref[2], misc[2], num_objs, num_weighed = 0,
		near_miss[2][5], known[2][5], obj_count, none_weighed = YES;
	char 	classes[2][5][COMMAND_LENGTH], weigh_these[OBJ_STR_LENGTH],
		same_obj[COMMAND_LENGTH], *capped,
		buffer[OBJ_STR_LENGTH];

	weigh_these[0] = 0;
	object_manip(WEIGH_OBJ,flg_ptr,inv_ptr,known,near_miss,classes,obj_num,obj_ref,weigh_these,misc);
	if (misc[0] == -1)
		return;
	obj_count = misc[1];
	strcpy(weigh_these,count_obj(weigh_these,WEIGH_OBJ));
	obj_num[OBJECT] = front_obj(weigh_these);
	while (obj_num[OBJECT]) {
        	num_objs = front_obj(weigh_these);
		if (num_objs > obj_count)
			num_objs = obj_count;
		buffer[0] = same_obj[0] = 0;
		mass = objects[obj_num[OBJECT]].mass;                          /* Take the mass of the object. */
		if (objects[obj_num[OBJECT]].contains_inside)                  /* add whatever it contains. */
			mass += objects[obj_num[OBJECT]].contains_inside->contained_wt;
		add_dig_to_string(same_obj,obj_num[OBJECT]);
		add_dig_to_string(same_obj,num_objs);
		show_obj("",same_obj,buffer,1,0);
		if (num_objs == 1)
			strcat(buffer," weighs ");
		else
			strcat(buffer," each weigh ");
		strcat(buffer,show_mass(mass));
		if (num_objs != 1) {
			strcat(buffer," giving a total of ");
			mass *= num_objs;
			strcat(buffer,show_mass(mass));
		}
		strcat(buffer,".\n");
		strcpy(buffer,capped = cap_first(buffer));
		free(capped);
		printf("%s",word_wrap(buffer));
		total_mass += mass;                                            /* Keep a running total for later on. */
		obj_count -= num_objs;
		num_weighed += num_objs;
		none_weighed = NO;					       /* Yes, we've weighed one. */
		if (obj_count == 0)                                            /* Have we weighed as many as required? */
			break;                                                 /* If so, stop here. */
                obj_num[OBJECT] = front_obj(weigh_these);                      /* Next object please. */
	}
	flg_ptr->prompt = YES;
	if (!none_weighed) {
		if (total_mass != mass) {
			strcpy(buffer,"The total weight of ");
			if (num_weighed == 2)
				strcat(buffer,"both");
			else {
				strcat(buffer,"all ");
				strcat(buffer,dig_to_word(num_weighed));
			}
			strcat(buffer," of the above objects is ");
			strcat(buffer,show_mass(total_mass));
			strcat(buffer,".\n");
                	printf("%s",word_wrap(buffer));
		}
		return;
	}
	manip_error(WEIGH_OBJ,OBJECT,classes,known,near_miss,obj_ref);         /* If nothing was weighed, display an error message. */
}

void where_object(struct flgs *flg_ptr, struct inv *inv_ptr, struct player_info *player_ptr)
{
	/*******************************************************************/
	/* Finds the location of an object, or group of objects and then   */
	/* displays it.                                                    */
	/*******************************************************************/

	int 	obj_num[2], obj_ref[2], misc[2], num_located = 0, num,
		near_miss[2][5], known[2][5], obj_count, none_located = YES;
	char 	classes[2][5][COMMAND_LENGTH], locate_these[OBJ_STR_LENGTH],
		print_buffer[COMMAND_LENGTH];

	locate_these[0] = 0;
	object_manip(WHERE_OBJ,flg_ptr,inv_ptr,known,near_miss,classes,obj_num,obj_ref,locate_these,misc);
	strcpy(locate_these,count_obj(locate_these,WHERE_OBJ));
	if (misc[0] == -1)
		return;
	obj_count = misc[1];
	obj_num[OBJECT] = front_obj(locate_these);
	num = front_obj(locate_these);
	while (obj_num[OBJECT]) {
		if (num > obj_count)
			num = obj_count;
		if (objects[obj_num[OBJECT]].name == NULL)
			break;
		printf("%i ",num);
		if (objects[obj_num[OBJECT]].loc_type == ROOM) {
			printf("in the room described as %s",rooms[objects[obj_num[OBJECT]].location].sdesc);
		} else if (objects[obj_num[OBJECT]].loc_type == CONTAINER) {
			printf("inside %s.\n",objects[objects[obj_num[OBJECT]].location].theword);
		} else if (objects[obj_num[OBJECT]].loc_type == HELD) {
			printf("carried by %s\n",player_ptr->name);
		}
		none_located = NO;
		obj_count -= num;
		num_located += num;
                obj_num[OBJECT] = front_obj(locate_these);
                num = front_obj(locate_these);
		if (obj_count == 0)
			break;
	}
	flg_ptr->prompt = YES;
	if (!none_located) {
		strcpy(print_buffer,"Your spell located ");
		strcat(print_buffer,dig_to_word(num_located));
		strcat(print_buffer," object");
		if (num_located != 1)
			strcat(print_buffer,"s");
		strcat(print_buffer, ".\n");
		printf("%s",word_wrap(print_buffer));
		return;
	}
	printf("Unfortunately, your spell fails to locate any objects.\n");
}
