/******************************************************/
/*         ROOMS - A room 'hanging' utility.          */
/*         V 1.7                                      */
/*         (c) October 1989. L.F. Holgate             */
/*                                                    */
/*         Persona file routines.                     */
/*                                                    */
/******************************************************/

#include <stdio.h>
#include <string.h>
#include <alloc.h>
#include <ctype.h>
#include <stdlib.h>
#include "rooms17.h"
#include "player.h"

extern char	*cap_first(char *in_file),
                *dig_to_word(int digit),
		*get_buffer(FILE *fp,int type),
		*get_new_command (void),
		*get_command (char *command_buffer);

extern struct player_idx *load_player(void);

void	set_flgs(struct flgs *flg_ptr),
	set_player(struct player_info *player_ptr),
	set_persona(struct player_info *player_ptr,struct player_idx *selected_persona),
	display_personae(struct player_idx *player_file);

char 	*get_race_name(int race_num),
	*get_class_name(int class_num, int sex),
	*get_level_name(int class_num, int level, int sex);

int     select_player(struct player_idx *player_file, struct player_info *player_ptr),
	pick_persona(struct player_idx *selected_persona);

void set_player(struct player_info *player_ptr)
{
	float	str_factor, str, siz, wt;

	player_ptr->siz = (player_ptr->height + player_ptr->weight)/2;
        str = player_ptr->str;
	siz = player_ptr->siz;
	wt = player_ptr->weight;
	str_factor = (float)(str/siz) * (str*(siz/wt))/100;
	while (str_factor > 10)
		str_factor = str_factor/10;
	player_ptr->max_wt = (str_factor*player_ptr->weight*445/(float)1.8); /* in kilos */
	player_ptr->wt_car = 0l;
	player_ptr->race_name = get_race_name(player_ptr->race);
	player_ptr->class_name = get_class_name(player_ptr->class,player_ptr->sex);
	player_ptr->level_name = get_level_name(player_ptr->class,player_ptr->level,player_ptr->sex);
}

int select_player(struct player_idx *player_file, struct player_info *player_ptr)
{
	struct player_idx  *selected_persona;
	int	choice, count = 1;

	selected_persona = player_file;
	clrscr();
	printf("The Camelot Computer Moderated Role Playing Game.\n\n");
	printf("Welcome to Camelot.\n");
	do {
		printf("\nThe following personae are available for you to use:\n\n");
		display_personae(player_file);
		for (choice = 0; choice == 0;choice = pick_persona(player_file));
	} while (choice == -3);
	if (choice > 0) {
		while (count != choice) {
			++count;
			selected_persona = selected_persona->next_player;
		}
		set_persona(player_ptr,selected_persona);
	} else if (choice == -1) {
		return(1);
	} else if (choice == -2) {
		return(1);
	}
	return(0);
}

void 	set_persona(struct player_info *player_ptr,struct player_idx *selected_persona)
{
		player_ptr->name = selected_persona->name;
		player_ptr->title = selected_persona->title;
		player_ptr->str = selected_persona->str;
		player_ptr->dex = selected_persona->dex;
		player_ptr->con = selected_persona->con;
		player_ptr->sta = selected_persona->sta;
		player_ptr->mag = selected_persona->mag;
		player_ptr->height = selected_persona->height;
		player_ptr->weight = selected_persona->weight;
		player_ptr->sex = selected_persona->sex;
		player_ptr->race = selected_persona->race;
		player_ptr->handedness = selected_persona->handedness;
		player_ptr->class = selected_persona->class;
		player_ptr->level = selected_persona->level;
		player_ptr->magic_exp = selected_persona->magic_exp;
		player_ptr->combat_exp = selected_persona->combat_exp;
		player_ptr->general_exp = selected_persona->general_exp;
}

int pick_persona(struct player_idx *player_file)
{
	char 	name[20];
	int	persona_num = 1;

	name[0] = '\0';
        while (strlen(name) == 0) {
		printf("%s",word_wrap("Type C to create a new persona, Q to quit, L to list personae available, or enter the name of the persona you wish to use: "));
		scanf("%s",&name);
	}
	getchar();
	name[0] = toupper(name[0]);
	if (!strcmp(name,"Q"))
		return(-1);
	else if (!strcmp(name,"C")) {
		printf("Create routines not in yet, use a word processor to edit PLAYER.PLA directly.\n");
		return(0); /* -2 */
	} else if (!strcmp(name,"L"))
		return(-3);
	while(player_file != NULL) {
		if (!strcmp(name,player_file->name))
			break;
		player_file = player_file->next_player;
		persona_num++;
	}
	if (player_file == NULL) {
		printf("You don't have a persona called that!\n");
		return(0);
	}
	return(persona_num);
}

void display_personae(struct player_idx *player_file)
{
	struct player_idx *current_player;

	current_player = player_file;
	while (current_player != NULL) {
		printf("%s the %s %s,",current_player->name,current_player->sex == 0 ? "male" : "female",get_race_name(current_player->race));
		printf(" a level %s %s.\n",dig_to_word(current_player->level),get_class_name(current_player->class,current_player->sex));
		current_player = current_player->next_player;
	}
	puts("");
}
char *get_level_name(int class_num, int level, int sex)
{
	switch (class_num) {
		case ADVENTURER :
			switch (level) {
				default :
					return("Adventurer");
			}
		case MERCENARY :
			switch (level) {
				case 1 :
					return("Recruit");
				case 2 :
					return("Fearless");
				case 3 :
					return("Worthy");
				case 4 :
					return("Warrior");
				case 5 :
					return("Yeoman-at-arms");
				case 6 :
					return("Spearman");
				case 7 :
					return("Swordsman");
				case 8 :
					return("Trooper");
				case 9 :
					return("Soldier");
				case 10 :
					return("Man-at-arms");
				case 11 :
					return("Hero");
				case 12 :
					return("Warder");
				case 13 :
					return("Guardsman");
				case 14 :
					return("Guardian");
				case 15 :
					return("Superhero");
				case 16 :
					return("Ranger");
				case 17 :
					return("Campaigner");
				case 18 :
					return("Advocate");
				case 19 :
					return("Veteran");
				case 20 :
					return("Mercenary");
			}
		case SQUIRE :
                        switch (level) {
				case 1 :
					return("Swordsman");
				case 2 :
					return("Protector");
				case 3 :
					return("Stalwart");
				case 4 :
					return("Valiant");
				case 5 :
					return("Master Swordsman");
				case 6 :
					return("Dauntless");
				case 7 :
					return("Courageous");
				case 8 :
					return("Contender");
				case 9 :
					return("Custodian");
				case 10 :
					return("Squire");
			}
		case KNIGHT :
			switch (level) {
				case 1 :
					return("Gallant");
				case 2 :
					return("Lancer");
				case 3 :
					return("Defender");
				case 4 :
					return("Justicar");
				case 5 :
					return("Champion");
				case 6 :
					return("Crusader");
				case 7 :
					return("Conqueror");
				case 8 :
					return("Lord");
				case 9 :
					return("Legend");
				case 10 :
					return("Knight");
			}
		case APPRENTICE :
                        switch (level) {
				case 1 :
					return("Trainee");
				case 2 :
					return("Student");
				case 3 :
					return("Pupil");
				case 4 :
					return("Novice");
				case 5 :
					return("Apprentice");
				case 6 :
					return("Dabbler");
				case 7 :
					return("Novitiate");
				case 8 :
					return("Neophyte");
				case 9 :
					return("Apt");
				case 10 :
					return("Adept");
			}
		case WHITE_MAGICIAN:
                        switch (level) {
				case 1 :
					if (!sex)
						return("Conjurer");
					else
						return("Conjuress");
				case 2 :
					if (!sex)
						return("Spellbinder");
					else
						return("Spellbindress");
				case 3 :
					return("Cabalist");
				case 4 :
					return("Magician");
				case 5 :
					if (!sex)
						return("Master of Magic");
					else
						return("Mistress of Magic");
				case 6 :
					return("Symbolist");
				case 7 :
					if (!sex)
                                        	return("Enchanter");
					else
						return("Enchantress");
				case 8 :
					if (!sex)
                                        	return("Sorcerer");
					else
						return("Sorcereress");
				case 9 :
					if (!sex)
						return("Wizard");
					else
						return("Witch");
				case 10 :
					return("Arch-Mage");
			}
		case ILLUSIONIST:
			switch (level) {
				case 1 :
					return("Beguiler");
				case 2 :
					return("Mesmerist");
				case 3 :
					return("Apparitionist");
				case 4 :
					return("Delusionist");
				case 5 :
					return("Imager");
				case 6 :
					return("Phantasmalist");
				case 7 :
					return("Aberrationist");
				case 8 :
					return("Thaumaturgist");
				case 9 :
					return("Illusionist");
				case 10 :
					return("Arch-Imager");
			}
		case BLACK_MAGICIAN :
                	switch (level) {
				case 1 :
					return("Occultist");
				case 2 :
					return("Infidel");
				case 3 :
					return("Heretic");
				case 4 :
					return("Bedeviler");
				case 5 :
					return("Bewitcher");
				case 6 :
					if (!sex)
						return("Debaucher");
					else
						return("Seductress");
				case 7 :
					return("Deviant");
				case 8 :
					return("Demonist");
				case 9 :
					return("Diabolist");
				case 10 :
					if (!sex)
						return("Warlock");
					else
						return("Witch");
			}
		case NECROMANCER :
        		switch (level) {
				case 1 :
					return("Fiend");
				case 2 :
					return("Animator");
				case 3 :
					return("Evoker");
				case 4 :
					return("Desecrator");
				case 5 :
					return("Violator");
				case 6 :
					return("Spirit Raiser");
				case 7 :
					return("Summoner");
				case 8 :
					return("Pogromist");
				case 9 :
					if (!sex)
						return("Psychomancer");
					else
						return("Psychomancess");
				case 10 :
					if (!sex)
						return("Necromancer");
					else
						return("Necromancess");
			}
	}
	return("ERROR");
}

char *get_class_name(int class_num, int sex)
{
	switch (class_num) {
		case ADVENTURER :
			return("Adventurer");
		case MERCENARY :
			return("Mercenary");
		case SQUIRE :
			return("Squire");
		case KNIGHT :
			return("Knight");
		case APPRENTICE :
			return("Apprentice Magician");
		case WHITE_MAGICIAN:
			return("White Magician");
		case ILLUSIONIST:
			return("Illusionist");
		case BLACK_MAGICIAN :
			return("Black Magician");
		case NECROMANCER :
			if (!sex)
				return("Necromancer");
			else
				return("Necromancess");
	}
	return("ERROR");
}

char *get_race_name(int race_num)
{
	switch (race_num) {
		case HUMAN :
			return("Human");
		case ELF :
			return("Elf");
		case DWARF :
			return("Dwarf");
		case GNOME :
			return("Gnome");
		case GIANT :
			return("Giant");
		case GOBLIN :
			return("Goblin");
		case ORC :
			return("Orc");
		case OGRE :
			return("Ogre");
		case FAIRY :
			return("Fairy");
	}
	return("ERROR");
}
