/******************************************************/
/*         ROOMS - A room 'hanging' utility.          */
/*         V 1.7                                      */
/*         (c) October 1989. L.F. Holgate             */
/*                                                    */
/*         main() and the main control loop.          */
/*                                                    */
/******************************************************/

#include <stdio.h>
#include <string.h>
#include <alloc.h>
#include <ctype.h>
#include <stdlib.h>
#include "rooms17.h"
#include "player.h"

extern char	*cap_first(char *in_file),
                *dig_to_word(int digit),
		*get_buffer(FILE *fp,int type),
		*get_new_command (void),
		*get_command (char *command_buffer);

extern struct player_idx *load_player(void);

void 	main (int argc,char *argv[]),
	title (void),
	wander (struct player_idx *player_file),
	set_flgs(struct flgs *flg_ptr),
	error(int err_num);

int	init_all(struct flgs *flg_ptr,struct inv *inv_ptr, struct player_info *player_ptr,struct player_idx *player_file);


struct rm_file rooms [MAX_ROOMS];
struct vo_file vocab [MAX_WORDS];
struct obj objects [MAX_OBJECTS];

char *messages[MAX_MESSAGE];

void main (int argc, char *argv[])
{
	struct player_idx *player_file;

	read_vocab();
        player_file = load_player();
	if (argc > 1) {
		read_rooms(argv [1]);
		read_messages();
		read_objs();
		wander(player_file);
	}
	for(;;) {
		menu();
		if (rooms[1].sdesc != NULL)
			wander(player_file);
	}
}

void wander(struct player_idx *player_file)
{
	struct flgs 		flag, *flg_ptr;
	struct inv  		inventory, *inv_ptr;
	struct player_info 	player, *player_ptr;

	int     cmd_code[2];
	char    static command_buffer [OBJ_STR_LENGTH],
		command [COMMAND_LENGTH], *capped;

	if (init_all(flg_ptr = &flag,inv_ptr = &inventory,player_ptr = &player,player_file) == 1)
		return;
	command_buffer [0] = 0;
	clrscr();
	for(;;) {
		if (flag.old_num != flag.room_num) {
			flag.moved = YES;
			flag.old_num = flag.room_num;
		}
		if (flag.moved) {
			display_room (flg_ptr);
			flag.moved = NO;
		}
		if (flag.prompt) {
			printf(PROMPT);
			flag.prompt = NO;
		}
		if (kbhit()) {
			if (strlen(strcat(command_buffer,get_new_command())) < 1)
				flag.prompt = YES;
		} else if (strlen(command_buffer) != 0)
			puts("");
		if (strlen(command_buffer) != 0) {
			strcpy(command,get_command(command_buffer));
			process_command(flg_ptr,command);
			cmd_code[0] = flag.commands [0][0];
			cmd_code[1] = flag.commands [0][1];

			if MOVE_CMD
				make_a_move(flg_ptr);
			else if LOOK_CMD
				take_a_look(flg_ptr);
			else if BRIEF_TOGGLE
				brief_toggle(flg_ptr);
			else if SCORE_CMD
				scores(flg_ptr,player_ptr);
			else if QUIT_CMD {
				if (inv_ptr->first) {
					process_command(flg_ptr,"dr all");
					drop_object(flg_ptr,inv_ptr,player_ptr);
				}
				puts("\nCheerio!");
				delay(500);
				return;
			} else if EXITS_CMD
				display_exits(flg_ptr);
			else if DEBUG_TOGGLE
				debug_toggle(flg_ptr);
			else if MAGICAL_CMD
				teleport(flg_ptr);
			else if HELP_CMD
				display_cmds(flg_ptr);
			else if GET_CMD
				get_object(flg_ptr,inv_ptr,player_ptr);
			else if INV_CMD
				show_inv(flg_ptr,inv_ptr);
			else if DROP_CMD
				drop_object(flg_ptr,inv_ptr,player_ptr);
			else if OPEN_CMD
				open_close(flg_ptr,inv_ptr,OPEN_OBJ);
			else if CLOSE_CMD
                        	open_close(flg_ptr,inv_ptr,CLOSE_OBJ);
			else if EMPTY_CMD
				empty_object(flg_ptr,inv_ptr,player_ptr);
			else if WHERE_CMD
				where_object(flg_ptr,inv_ptr,player_ptr);
			else if WEIGH_CMD
				weigh_object(flg_ptr,inv_ptr);
			else if EXAMINE_CMD
				examine_object(flg_ptr,inv_ptr);
			else if WHO_CMD
				who(flg_ptr);
			/*else if TOUCH_CMD */
			else if (cmd_code[0] == 0) {
				printf("%s?\n",capped = cap_first(dig_to_word(cmd_code[1])));
				free(capped);
				flag.prompt = YES;
			} else if (cmd_code[0] != -1 && cmd_code [1] != -1)   {
				printf("Hmm, I'm sure I should understand that, but I don't!\n");
				flag.prompt = YES;
			}
		}
	}
}

int init_all(struct flgs *flg_ptr,struct inv *inv_ptr, struct player_info *player_ptr,struct player_idx *player_file)
{
	set_flgs(flg_ptr);
	inv_ptr->first = inv_ptr->last = 0;
	if (select_player(player_file,player_ptr) == 1)
		return(1);
	set_player(player_ptr);
	return(0);
}

void set_flgs(struct flgs *flg_ptr)
{
	flg_ptr->room_num = flg_ptr->old_num = 1;
	flg_ptr->moved = flg_ptr->prompt = flg_ptr->can_see = YES;
	flg_ptr->brief_state = VERBOSE;
	flg_ptr->debug_state = OFF;
}

void title (void)
{
	puts("\n\t\t\t\tROOMS!\n\t\t\t\t V1.7\n\n\t\t  The room display utility (that grew)."
	     "\n\n\t\t      (c) October 1989, L.F.Holgate.\n");
}

void error(int err_num)
{
	printf("Error %i:\nFailure in memory allocation routines.",err_num);
	exit(1);
}
